//*----------------------------------------------------------------------------
//* LCD.C
//* ROUTINEN ZUR ANSTEUERUNG DES DOGM128
//*----------------------------------------------------------------------------
#include "tools.h"



//*----------------------------------------------------------------------------
#include "font/SystemFont.h"
#include "font/HeaderFont.h"
#include "font/BigFont.h"
//*----------------------------------------------------------------------------
// LCD-Variablen
//*----------------------------------------------------------------------------
TLCD lcd;

// LCD DOGM128-6 INITIALISIEREN
// #define LCD_WIDTH   128// DOGM128 siehe lcd.dogm128.h
// #define LCD_HEIGHT  64
#if DISPLAY_TYPE == 128

static const PROGMEM uint8_t Lcd_InitData[] = {
  0x40,   // Display Start Line 0
  0xA1,   // Display Normal / reverse A0 
  0xC0,	  // Display Normal / reverse C8
  0xA6,
  0xA2,   // Set Bias
  0x2F,   // Power Control set
  0xF8,   // Booster Ratio set
  0x00,   // ""
  0x27,   // Contrast set
  0x81,
  0x16,
  0xAC,   // Static Indicator set
  0x00,   // ""
  0xAF    // Display on/off
};
#endif

 #if DISPLAY_TYPE == 102
  static const PROGMEM uint8_t Lcd_InitData[] = {
  0x40,   	// Display Start Line 0
  0xA1,   	// Display Normal / reverse A0 
  0xC0,	  	// Display Normal / reverse C8
  0xA4,		// Disable->Set ALL Pixel to ON	
  0xA6,   	// Display inverse off
  0xA2,   	// Set Bias1/9
  0x2F,   	// Booster, Regulator and Follower on
  0x10,   	// Contrast set
  0x81,		// dito
  0x10,		// dito
  0xFA,   	// Set Temperature compensation
  0x90,   	// dito
  0xAF    	// Display on
};
#endif
  
 
  
  



//*----------------------------------------------------------------------------
void Lcd_Init(void)
{
  uint8_t i;

  Lcd_Reset_HI();   // Display Reset-Leitung High
  Lcd_Select();     // Display selektieren
  Lcd_A0_LO();      // Command-Mode
  for(i = 0; i < sizeof(Lcd_InitData); i++) {
    Spi_Io(pgm_read_byte(&Lcd_InitData[i]));
  }
  Lcd_Deselect();   // Display deselektieren
}
//*----------------------------------------------------------------------------
// Contrast und Direction setzen
//*----------------------------------------------------------------------------
void Lcd_SetContDir(uint8_t cont, bool dir)
{
  Lcd_Select();     // Display selektieren
  Lcd_A0_LO();      // Command-Mode
  Spi_Io(0x81);     // 0x81 = Set Contrast
  Spi_Io(cont);     // Contrast value
  if(dir == ORIENTATION_REVERSE) {
    Spi_Io(0xA1);     // ADC set (A1/A0)
    Spi_Io(0xC0);     // COM output scan direction COM0..31 (C0/C8)
  } else {
    Spi_Io(0xA0);     // ADC set (A1/A0)
    Spi_Io(0xC8);     // COM output scan direction COM0..31 (C0/C8)
  }
  Lcd_Deselect();   // Display deselektieren
  lcd.invers = false;     // Display nicht invers
}
//*----------------------------------------------------------------------------
// Display aus Mirror-Buffer anzeigen
//*----------------------------------------------------------------------------
void Lcd_Show(void)
{
  volatile uint8_t x, y;

  Lcd_Select();       // Display selektieren
  for(y = 0; y < 8; y++) {
    Lcd_A0_LO();      // Command-Mode
    Spi_Io(y + 0xB0); // Row select
    Spi_Io(0x10);     // Column select high
    Spi_Io(0x00);     // Column select low
    Lcd_A0_HI();      // Data-Mode
    for(x = 0; x < 128; x++) { Spi_Io(lcd.Mirror[x][y]); }
  }
  Lcd_Deselect();     // Display deselektieren
}
//*----------------------------------------------------------------------------
// Display (Mirror) lschen
// danach unbedingt Lcd_Show() aufrufen
//*----------------------------------------------------------------------------
void Lcd_Clear(void)
{
  uint16_t i;
  uint8_t  color = 0x00;
  uint8_t  *ptr = (uint8_t *)&lcd.Mirror;

  if(lcd.invers) { color = 0xFF; }
  for(i = 0; i < sizeof(lcd.Mirror); i++) { *ptr++ = color; }
}
//*----------------------------------------------------------------------------
// Pixel setzen
// x = 0..128, y = 0..64
//*----------------------------------------------------------------------------
void Lcd_SetPixel(uint8_t x, uint8_t y)
{
  if((y >= LCD_HEIGHT) || (x >= LCD_WIDTH)) return;
  if(lcd.invers) {
    lcd.Mirror[x][y / 8] &= ~(1 << (y & 0x07));
  } else {
    lcd.Mirror[x][y / 8] |= (1 << (y & 0x07));
  }
}
//*----------------------------------------------------------------------------
// Pixel lschen
// x = 0..128, y = 0..64
//*----------------------------------------------------------------------------
void Lcd_ClrPixel(uint8_t x, uint8_t y)
{
  if((y >= LCD_HEIGHT) || (x >= LCD_WIDTH)) return;
  if(lcd.invers) {
    lcd.Mirror[x][y / 8] |= (1 << (y & 0x07));
  } else {
    lcd.Mirror[x][y / 8] &= ~(1 << (y & 0x07));
  }
}
//*----------------------------------------------------------------------------
// Horizontale Linie zeichnen
// x = 0..128, y = 0..64, w = Lnge
//*----------------------------------------------------------------------------
void Lcd_Hline(uint8_t x, uint8_t y, uint8_t w)
{
  uint8_t xp;

  for(xp = x; xp <= x + w - 1; xp++) { Lcd_SetPixel(xp, y); }
}
//*----------------------------------------------------------------------------
// Vertikale Linie zeichnen
// x = 0..128, y = 0..64, h = Hhe
//*----------------------------------------------------------------------------
void Lcd_Vline(uint8_t x, uint8_t y, uint8_t h)
{
  uint8_t yp;

  for(yp = y; yp <= y + h - 1; yp++) { Lcd_SetPixel(x, yp); }
}
//*----------------------------------------------------------------------------
// Bereich lschen
//*----------------------------------------------------------------------------
void Lcd_Erase(uint8_t x, uint8_t y, uint8_t w, uint8_t h)
{
  uint8_t xp, yp;

  for(xp = x; xp <= x + w - 1; xp++) {
    for(yp = y; yp <= y + h - 1; yp++) {
      Lcd_ClrPixel(xp, yp);
    }
  }
}
//*----------------------------------------------------------------------------
// DISP-Rectangle
// x = 0..128, y = 0..64, w = Breite, h = Hhe, mode = FRAMED/FILLED
//*----------------------------------------------------------------------------
void Lcd_Rectangle(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t mode)
{
  uint8_t xp, yp;

  if(mode == FRAMED) {
    for(xp = x; xp <= x + w - 1; xp++) {
      Lcd_SetPixel(xp, y);
      Lcd_SetPixel(xp, y + h - 1);
    }
    for(yp = y + 1; yp < y + h - 1; yp++) {
      Lcd_SetPixel(x, yp);
      Lcd_SetPixel(x + w - 1, yp);
    }
  } else {
    for(xp = x; xp <= x + w - 1; xp++) {
      for(yp = y; yp <= y + h - 1; yp++) {
        Lcd_SetPixel(xp, yp);
      }
    }
  }
}
//*----------------------------------------------------------------------------
// Round Rectangle
//*----------------------------------------------------------------------------
void Lcd_RoundRectangle(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t mode)
{
  uint8_t xp, yp;

  if(mode == FRAMED) {
    for(xp = x + 2; xp <= x + w - 3; xp++) {
      Lcd_SetPixel(xp, y);
      Lcd_SetPixel(xp, y + h - 1);
    }
    Lcd_SetPixel(x + 1,     y + 1);
    Lcd_SetPixel(x + w - 2, y + 1);
    Lcd_SetPixel(x + 1,     y + h - 2);
    Lcd_SetPixel(x + w - 2, y + h - 2);
    for(yp = y + 2; yp <= y + h - 3; yp++) {
      Lcd_SetPixel(x, yp);
      Lcd_SetPixel(x + w - 1, yp);
    }
  } else {
    Lcd_Hline(x + 2, y,     w - 4);
    Lcd_Hline(x + 1, y + 1, w - 2);
    for(yp = y + 2; yp <= y + h - 3; yp++) {
      Lcd_Hline(x, yp, w);
    }
    Lcd_Hline(x + 1, yp,     w - 2);
    Lcd_Hline(x + 2, yp + 1, w - 4);
  }
}
//*----------------------------------------------------------------------------
// Fortschrittsbalken anzeigen
//*----------------------------------------------------------------------------
void Lcd_Bar(uint8_t x, uint8_t y, uint8_t w, uint8_t max, uint8_t pos)
{
  uint16_t len = ((uint16_t)(w - 4) * (uint16_t)pos) / (uint16_t)max;

  Lcd_RoundRectangle(x, y, w, 7, 0);
  lcd.invers = true;
  Lcd_Rectangle(x + 2, y + 2, w - 4, 3, FILLED);
  lcd.invers = false;
  Lcd_Rectangle(x + 2, y + 2, len, 3, FILLED);
}
//*----------------------------------------------------------------------------

//*----------------------------------------------------------------------------
// FONT-ROUTINEN
//*----------------------------------------------------------------------------
//*----------------------------------------------------------------------------
// Font Init
// Setzt die Font-Headerdaten
//*----------------------------------------------------------------------------
void Font_Init(uint8_t *ptr)
{
  lcd.Char_First  = pgm_read_byte(&((TFONTHEAD *)ptr)->Font_FirstChar);
  lcd.Char_Last   = pgm_read_byte(&((TFONTHEAD *)ptr)->Font_LastChar);
  lcd.Font_Height = pgm_read_byte(&((TFONTHEAD *)ptr)->Font_Height);
  lcd.fontwidth =  ptr + sizeof(TFONTHEAD);
  lcd.fontdata =   lcd.fontwidth + (lcd.Char_Last - lcd.Char_First + 1);

  lcd.X_Pos = 0;
  lcd.X_Max = LCD_WIDTH - 1;    // (131)
  lcd.Y_Pos = 0;
}
//*----------------------------------------------------------------------------
// Position fr Textausgabe setzen
//*----------------------------------------------------------------------------
void Font_SetPos(uint8_t x, uint8_t y)
{
  lcd.X_Pos = x;
  lcd.Y_Pos = y;
}
//*----------------------------------------------------------------------------
// Fordert die Breite in pixeln den genschten Zeichens an (0 = Error)
//*----------------------------------------------------------------------------
uint8_t Font_CharWidth(char c)
{
  if(c < lcd.Char_First) { return 0; }
  if(c > lcd.Char_Last)  { return 0; }
  return pgm_read_byte(&lcd.fontwidth[c - lcd.Char_First]);
}
//*----------------------------------------------------------------------------
// Breite des Textes in Pixeln errechnen
//*----------------------------------------------------------------------------
uint16_t Font_TextWidth(char *ptr, uint8_t mode)
{
  uint16_t width = 0;
  char c;

  while(true) {
    if(mode & TXT_MODE_FLASH) { c = pgm_read_byte(ptr); }
                         else { c = *ptr; }
    if(c == 0) { break; }
    width += Font_CharWidth(c) + 1;
    ptr++;
  }
  return width - 1;
}
//*----------------------------------------------------------------------------
// Zeichen anzeigen mit dem aktuellen Zeichensatz (aus Font_init)
//*----------------------------------------------------------------------------
void Font_DrawChar(char c)
{
  uint8_t x, y, i = 0;
  uint16_t idx = 0;
  uint8_t BitCount;
  uint8_t width = Font_CharWidth(c);
  uint8_t FData;

  if(width == 0) { return; }
  if((lcd.X_Pos + width) > lcd.X_Max) { return; }

  c -= lcd.Char_First;
  while(i < c) { idx += pgm_read_byte(&lcd.fontwidth[i++]); }
  idx = idx * lcd.Font_Height;
  BitCount = idx & 0x07;
  idx >>= 3;
  FData = pgm_read_byte(&lcd.fontdata[idx++]) >> BitCount;

  for(x = 0; x < width; x++) {
    for(y = 0; y < lcd.Font_Height; y++) {
      if(BitCount > 7) {
        FData = pgm_read_byte(&lcd.fontdata[idx++]);
        BitCount = 0;
      }
      if(FData & 0x01) { Lcd_SetPixel(lcd.X_Pos + x, lcd.Y_Pos + y); }
                  else { Lcd_ClrPixel(lcd.X_Pos + x, lcd.Y_Pos + y); }
      FData >>= 1;
      BitCount++;
    }
  }
  lcd.X_Pos += width + 1;
	//Lcd_Show(d);
}
//*----------------------------------------------------------------------------
// Text ausgeben an x,y
//*----------------------------------------------------------------------------
void Font_TextOut(char *ptr, uint8_t x, uint8_t y, uint8_t mode)
{
  char c;

  lcd.X_Pos = x; lcd.Y_Pos = y;
  while(true) {
    if(mode & TXT_MODE_FLASH) { c = pgm_read_byte(ptr); }
                         else { c = *ptr; }
    if(c == 0) { break; }
    Font_DrawChar(c);
    ptr++;
  }
}
//*----------------------------------------------------------------------------
// HEX-Nibble ausgeben
//*----------------------------------------------------------------------------
void Font_HexNibble(uint8_t b)
{
  uint8_t c1 = (b & 0x0F);
  if(c1 < 10) { Font_DrawChar(c1 + '0'); }
         else { Font_DrawChar(c1 + (0x1A - 10)); }
}



//*----------------------------------------------------------------------------
// Hex32 ausgeben
//*----------------------------------------------------------------------------

// print 16-bit hex value
void Font_Hex32(uint32_t b)
{


Font_Hex16(b >> 16);
Font_Hex16(b);
}


//*----------------------------------------------------------------------------
// Hex16 ausgeben
//*----------------------------------------------------------------------------

// print 16-bit hex value
void Font_Hex16(uint16_t b)
{
uint8_t result_l;
uint8_t result_h;
    result_l = b & 0xFF;
    result_h = b >> 8;

Font_Hex(result_h);
Font_Hex(result_l);
}


//*----------------------------------------------------------------------------
// Hex ausgeben
//*----------------------------------------------------------------------------
void Font_Hex(uint8_t b)
{
  asm volatile("swap %0" : "=r" (b) : "0" (b));
  Font_HexNibble(b);
  asm volatile("swap %0" : "=r" (b) : "0" (b));
  Font_HexNibble(b);
}
//*----------------------------------------------------------------------------
// Dezimalwert 3-stellig anzeigen 0 - 255
//*----------------------------------------------------------------------------
void Font_Dez3(uint8_t b)
{
  uint8_t c, d;

  c = 0;
  while(b >= 100) { b -= 100; c++; }
  if(c != 0) { Font_DrawChar(c + '0'); }
  d = 0;
  while(b >= 10) { b -= 10; d++; }
  if((c !=0) || (d != 0)) { Font_DrawChar(d + '0'); }
  Font_DrawChar(b + '0');
}
//*----------------------------------------------------------------------------
// Text ausgeben
// Links/Rechtsbndig/Zentriert, Invertiert, Flash/Ram
//*----------------------------------------------------------------------------
void Font_TextPrint(char *ptr, uint8_t x, uint8_t y, uint8_t w, uint8_t mode)
{
  lcd.X_Max = x + w - 1;
  lcd.invers = !(mode & TXT_MODE_INVERS);
  Lcd_Rectangle(x, y, w, lcd.Font_Height, FILLED);
  lcd.invers = mode & TXT_MODE_INVERS;
  if(mode & TXT_MODE_CENTER) {
    lcd.X_Pos = x + (w >> 1) - (Font_TextWidth(ptr, mode) >> 1);
  } else if(mode & TXT_MODE_RIGHT) {
    lcd.X_Pos = x + w - Font_TextWidth(ptr, mode) - 1;
  } else {
    lcd.X_Pos = x;
  }
  Font_TextOut(ptr, lcd.X_Pos, y, mode);
  lcd.invers = false;
  lcd.X_Max = LCD_WIDTH - 1;
}
//*----------------------------------------------------------------------------
// TEXT FORMATIERT MIT UMBRUCH AUSGEBEN (nur aus Flash)
//*----------------------------------------------------------------------------
void Font_TextFormat(char *ptr, uint8_t x, uint8_t y, uint8_t w)
{
  char c;
  uint8_t p = 0, i;
  uint8_t cw, space;

  while(true) {
    Font_SetPos(x, y);
    cw = 0;
    space = p;
    for(i = p;; i++) {
      c = pgm_read_byte(&ptr[i]);
      if(c == 0) { space = i; break; }
      cw += Font_CharWidth(c) + 1;
      if(cw >= w) {
        if(space == p) { space = i - 1; }
        break;
      }
      if((c == ' ') || (c == '-') || (c == '/') || (c == '=')) { space = i; }
    }

    for(i = p; i <= space; i++) { Font_DrawChar(pgm_read_byte(&ptr[i])); }
    if(c == 0) { break; }
    p = (space + 1);
    y += 8;
    if(y >= LCD_HEIGHT) { break; } // Abbruch bei zu langem Text
  }
}

//*----------------------------------------------------------------------------
// *** rprintfNum ***
// special printf for numbers only
// see formatting information below
//	Print the number "n" in the given "base"
//	using exactly "numDigits"
//	print +/- if signed flag "isSigned" is TRUE
//	use the character specified in "padchar" to pad extra characters
//
//	Examples:
//	uartPrintfNum(10, 6,  TRUE, ' ',   1234);  -->  " +1234"
//	uartPrintfNum(10, 6, FALSE, '0',   1234);  -->  "001234"
//	uartPrintfNum(16, 6, FALSE, '.', 0x5AA5);  -->  "..5AA5"
//*----------------------------------------------------------------------------
void rprintfNum(char base, char numDigits, char isSigned, char padchar, long n)
{
	// define a global HexChars or use line below
	//static char HexChars[16] = "0123456789ABCDEF";
	char *p, buf[32];
	unsigned long x;
	unsigned char count;

	// prepare negative number
	if( isSigned && (n < 0) )
	{
		x = -n;
	}
	else
	{
	 	x = n;
	}

	// setup little string buffer
	count = (numDigits-1)-(isSigned?1:0);
  	p = buf + sizeof (buf);
  	*--p = '\0';
	
	// force calculation of first digit
	// (to prevent zero from not printing at all!!!)
	*--p = hexchar(x%base); x /= base;
	// calculate remaining digits
	while(count--)
	{
		if(x != 0)
		{
			// calculate next digit
			*--p = hexchar(x%base); x /= base;
		}
		else
		{
			// no more digits left, pad out to desired length
			*--p = padchar;
		}
	}

	// apply signed notation if requested
	if( isSigned )
	{
		if(n < 0)
		{
   			*--p = '-';
		}
		else if(n > 0)
		{
	   		*--p = '+';
		}
		else
		{
	   		*--p = ' ';
		}
	}

	// print the string right-justified
	count = numDigits;
	while(count--)
	{
		Font_DrawChar(*p++);
	}
}

#ifdef RPRINTF_FLOAT

//*----------------------------------------------------------------------------
//*** rprintfFloat ***floating-point print
//*----------------------------------------------------------------------------


void rprintfFloat(char numDigits, double x)
{
	unsigned char firstplace = FALSE;
	unsigned char negative;
	unsigned char i, digit;
	double place = 1.0;
	
	// save sign
	negative = (x<0);
	// convert to absolute value
	x = (x>0)?(x):(-x);
	
	// find starting digit place
	for(i=0; i<15; i++)
	{
		if((x/place) < 10.0)
			break;
		else
			place *= 10.0;
	}
	// print polarity character
	if(negative)
		Font_DrawChar('-');
	else
		Font_DrawChar('+');

	// print digits
	for(i=0; i<numDigits; i++)
	{
		digit = (x/place);

		if(digit | firstplace | (place == 1.0))
		{
			firstplace = TRUE;
			Font_DrawChar(digit+0x30);
		}
		else
			Font_DrawChar(' ');
		
		if(place == 1.0)
		{
			Font_DrawChar('.');
		}
		
		x -= (digit*place);
		place /= 10.0;
	}
}
#endif

void rprintfStr(char str[])
{
	// send a string stored in RAM
	// check to make sure we have a good pointer
	if (!str) return;

	// print the string until a null-terminator
	while (*str)
		Font_DrawChar(*str++);
}
void rprintfStrLen(char str[], unsigned int start, unsigned int len)
{
	register int i=0;

	// check to make sure we have a good pointer
	if (!str) return;
	// spin through characters up to requested start
	// keep going as long as there's no null
	while((i++<start) && (*str++));
//	for(i=0; i<start; i++)
//	{
//		// keep steping through string as long as there's no null
//		if(*str) str++;
//	}

	// then print exactly len characters
	for(i=0; i<len; i++)
	{
		// print data out of the string as long as we haven't reached a null yet
		// at the null, start printing spaces
		if(*str)
			Font_DrawChar(*str++);
		else
			Font_DrawChar(' ');
	}

}


//*----------------------------------------------------------------------------
// Fonts Init
//*----------------------------------------------------------------------------
void Font_SystemFont(void) { Font_Init((uint8_t *)&SystemFont); }
void Font_HeaderFont(void) { Font_Init((uint8_t *)&HeaderFont); }
void Font_BigFont(void)    { Font_Init((uint8_t *)&BigFont); }
//*----------------------------------------------------------------------------

