//*----------------------------------------------------------------------------
//* LCD.H
//*----------------------------------------------------------------------------
#ifndef __LCD_H
#define __LCD_H
//*----------------------------------------------------------------------------
#include "tools.h"
#include "spi.h"
//*----------------------------------------------------------------------------
// KONSTANTEN
//*----------------------------------------------------------------------------
#define ORIENTATION_NORMAL  false
#define ORIENTATION_REVERSE true

#define DISPLAY_TYPE  128
// #define DISPLAY_TYPE  102


//*----------------------------------------------------------------------------
#if DISPLAY_TYPE == 102
 #define LCD_WIDTH   102   // DOGS102
 #define LCD_HEIGHT  64
#endif 
//*----------------------------------------------------------------------------

//*----------------------------------------------------------------------------
#if DISPLAY_TYPE == 128
 #define LCD_WIDTH   128   // DOGM128
 #define LCD_HEIGHT  64
#endif
//*----------------------------------------------------------------------------
#define FILLED      true
#define FRAMED      false
//*----------------------------------------------------------------------------
#define TXT_MODE_LEFT     0x00
#define TXT_MODE_CENTER   0x01
#define TXT_MODE_RIGHT    0x02

#define TXT_MODE_NONINV   0x00
#define TXT_MODE_INVERS   0x04

#define TXT_MODE_RAM      0x00
#define TXT_MODE_FLASH    0x08

#define TXT_ADD_UP        0x10
#define TXT_ADD_DOWN      0x20
//*----------------------------------------------------------------------------
// MACROS
//*----------------------------------------------------------------------------

#define hexchar(x)	((((x)&0x0F)>9)?((x)+'A'-10):((x)+'0'))
#define  BIT(n)   (1 << (n))
#define Lcd_Select()        PORTB &= ~BIT(LCD_CS)
#define Lcd_Deselect()      PORTB |= BIT(LCD_CS)
#define Lcd_A0_LO()         PORTB &= ~BIT(LCD_A0)
#define Lcd_A0_HI()         PORTB |= BIT(LCD_A0)
#define Lcd_Reset_HI()      PORTB |= BIT(LCD_RST)
//*----------------------------------------------------------------------------
// VARIABLENDEFINITION
//*----------------------------------------------------------------------------
// Der Font-Header fr den Font-Editor von H.Reddmann
// Funktioniert nur mit unkomprimierten Fonts
//*----------------------------------------------------------------------------
typedef struct {
  uint8_t  Font_Size_Lo;       // Font size in Bytes LOW
  uint8_t  Font_Size_Hi;       // Font size in Bytes HIGH
  uint8_t  Font_Width;         // Font Width in Pixels
  uint8_t  Font_Height;        // Font Height in Pixels
  uint8_t  Font_BitsPerPixels; // Font Bits per Pixels (nur 1 erlaubt)
  uint8_t  Font_FirstChar;     // Font first char
  uint8_t  Font_LastChar;      // Font last char
} TFONTHEAD;
//*----------------------------------------------------------------------------
typedef struct {
  uint8_t   Mirror[128][8];    // Display Mirror-Buffer 4 x 132 Bytes
  bool invers;            // Flag fr invertiertes Display
  uint8_t *fontwidth;          // Zeiger auf die Font-Breitentabelle
  uint8_t *fontdata;           // Zeiger auf die Font-Daten
  uint8_t Char_First;          // First Char aus Header
  uint8_t Char_Last;           // Last Char aus Header
  uint8_t Font_Height;         // Font Height aus Header
  uint8_t X_Pos;               // Aktuelle Zeichenausgabe X-Position
  uint8_t X_Max;               // Rechte Begrenzung fr Zeichenausgabe
  uint8_t Y_Pos;               // Aktuelle Zeichenausgabe Y-Position
} TLCD;

extern TLCD lcd;
//*----------------------------------------------------------------------------
// FUNKTIONSPROTOTYPEN
//*----------------------------------------------------------------------------
void Lcd_Init(void);
void Lcd_Show(void);
void Lcd_Clear(void);
void Lcd_SetContDir(uint8_t cont, bool dir);
void Lcd_SetPixel(uint8_t x, uint8_t y);
void Lcd_ClrPixel(uint8_t x, uint8_t y);
void Lcd_Hline(uint8_t x1, uint8_t y1, uint8_t w);
void Lcd_Vline(uint8_t x, uint8_t y, uint8_t h);
void Lcd_Erase(uint8_t x, uint8_t y, uint8_t w, uint8_t h);
void Lcd_Rectangle(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t mode);
void Lcd_RoundRectangle(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t mode);
void Lcd_Bar(uint8_t x, uint8_t y, uint8_t w, uint8_t max, uint8_t pos);
//*----------------------------------------------------------------------------
void Font_Init(uint8_t *ptr);
void Font_DrawChar(char c);
uint8_t   Font_CharWidth(char c);
uint16_t  Font_TextWidth(char *ptr, uint8_t mode);
void Font_TextOut(char *ptr, uint8_t x, uint8_t y, uint8_t mode);
void Font_TextPrint(char *ptr, uint8_t x, uint8_t y, uint8_t w, uint8_t mode);
void Font_TextFormat(char *ptr, uint8_t x, uint8_t y, uint8_t w);
void Font_HexNibble(uint8_t b);
void Font_Hex16(uint16_t b);
void Font_Hex32(uint32_t b);
void rprintfFloat(char numDigits, double x);
void rprintfNum(char base, char numDigits, char isSigned, char padchar, long n);
void rprintfStr(char str[]);
void rprintfStrLen(char str[], unsigned int start, unsigned int len);
void Font_Hex(uint8_t b);
void Font_Dez3(uint8_t b);
void Font_SetPos(uint8_t x, uint8_t y);
//*----------------------------------------------------------------------------
void Font_SystemFont(void);
void Font_HeaderFont(void);
void Font_BigFont(void);
//*----------------------------------------------------------------------------
#endif
